
$ModulePath = "$ENV:SS_UTILITIES\PSModules\PLM-Dev\1.0.0\"
if ($PSVersionTable.PSVersion -lt [version]"5.1") {
    Import-LocalizedData -BindingVariable "Config" -FileName (Join-Path -Path $ModulePath -ChildPath "PLM-Dev-Config.psd1")
}
else {
    $Config = Import-PowerShellDataFile -Path (Join-Path -Path $ModulePath -ChildPath "PLM-Dev-Config.psd1")
}



function Install-PSGaleryModule {
    [CmdletBinding()]
    param (
        [Parameter()]
        [string]
        $Module,
        [pscredential]
        $Credential
    )

    Set-PSRepository -Name PSGallery -PackageManagementProvider NuGet -Credential $Credential
    Install-Module $Module -Scope AllUsers -Force -Credential $Credential
    
}

function Mount-NetworkDrive {
    [CmdletBinding()]
    param (
        [Parameter()]
        [string]
        $DriveLetter,
        [string]
        $UNCPath,
        [pscredential]
        $Credential
    )
    
    $MountParams = @{
        ArgumentList = @($DriveLetter, $UNCPath, $Credential.UserName, $Credential.GetNetworkCredential().Password)
        ScriptBlock = {
            Param($DriveLetter, $UNCPath, $UserName, $Password) 
            net use $DriveLetter $UNCPath /user:$UserName /persistent:yes $Password
        }
    }
    Invoke-Command @MountParams -OutBuffer $null | Out-Null
}

function Install-Scoop {
    [CmdletBinding()]
    param (
        [Parameter()]
        [string]
        $ScoopInstallerPath="https://get.scoop.sh"
    )

    Set-ExecutionPolicy RemoteSigned -Scope CurrentUser

    try {
        Invoke-RestMethod -Method Get -Uri $ScoopInstallerPath | Invoke-Expression
    }
    catch {

    }
}

function Install-ScoopBucket {
    [CmdletBinding()]
    param (
        [Parameter()]
        [string[]]
        $Bucket
    )

    $BucketParams = @{
        ArgumentList = @($Bucket)
        ScriptBlock = {
            Param($Bucket)
            scoop bucket add $Bucket
        }
    }
    Invoke-Command @BucketParams
}


function Install-ScoopPackage {
    [CmdletBinding()]
    param (
        [Parameter()]
        [string[]]
        $Package,
        [switch]
        $Global,
        [pscredential]
        $Credential
    )

    $ScriptBlock = {
        Param($Package)
        scoop install $Package
    }

    if ($Global) {
        $ScriptBlock = {
            Param($Package)
            scoop install $Package --global
        }
    }

    $InstallParams = @{
        ArgumentList = @($Pakage)
        ScriptBlock = $ScriptBlock
    }
    Invoke-Command @InstallParams -Credential $Credential
}

function Install-VSCodeExtension {
    [CmdletBinding()]
    param (
        [Parameter()]
        [string]
        $Extension
    )
    
    $ExtensionParams = @{
        ArgumentList = @($PluginName)
        ScriptBlock = {
            Param($PluginName)
            code.exe --install-extension $PluginName
        }
    }

    if ($(where.exe code | out-null).length) {
        Invoke-Command @ExtensionParams
    }

}




function Install-PLMDevEnvironment {
    [CmdletBinding()]
    param (
        [Parameter(HelpMessage="Will cause scoop to install all packages as Administrator, making the packages available to all users.")]
        [switch]
        $GlobalInstall
    )

    
    $Cred = $(Get-Credential -Message "Please provide your domain credentials to install packages as Administrator")

    
    try {
        $InstalledModules = Get-Module
        foreach ($Module in $Config.PrerequisitePSGaleryModules) {
            if ($InstalledModules.name -notcontains $Module) {
                Install-PSGaleryModule -Credential $Cred -Module $Module
            }
        }
    }
    catch {
        <#Do this if a terminating exception happens#>
    }

    try {
        $MappedDrives = Get-PSDrive -PSProvider FileSystem
        foreach ($NetworkDrive in $Config.DriveMappings) {
            if ($MappedDrives.Name -notcontains $NetworkDrive.DriveLetter) {
                Mount-NetworkDrive -Credential $Cred -DriveLetter $NetworkDrive.DriveLetter -UNCPath $NetworkDrive.UNCPath
            }
        }
    }
    catch {
        <#Do this if a terminating exception happens#>
    }



    try {
        if ($(Assert-AppInPath).Length)
        Install-Scoop
    }
    catch {
        <#Do this if a terminating exception happens#>
    }

    try {
        # Install pre-req packages
        foreach ($Package in $Config.PrerequisiteScoopPackages) {
            Install-ScoopPackage -Global $GlobalInstall -Pacakge $Package
        }
        
    }
    catch {
        <#Do this if a terminating exception happens#>
    }

    try {
        foreach ($Bucket in $Config.ScoopBuckets) {
            Install-ScoopBuckets -Bucket $Bucket
        }
    }
    catch {
        <#Do this if a terminating exception happens#>
    }

    try {
        foreach ($Package in $Config.ScoopPackages) {
            Install-ScoopPackage -Global $GlobalInstall -Pacakge $Package
        }
    }
    catch {
        <#Do this if a terminating exception happens#>
    }

    try {
        foreach ($Extension in $Config.VSCodeExtensions) {
            Install-VSCodeExtension -Extension $Extension
        }
    }
    catch {
        <#Do this if a terminating exception happens#>
    }
}




Export-ModuleMember -Function Install-PLMDevEnvironment